unit uDummyPort;

interface

uses
  Windows, Classes, Registry;

Type
  TCustomDummyComPort = class(TComponent)
  private
    FPort: string;
    FConnected: Boolean;
    FModemHandle: Cardinal;
    procedure SetPort(const Value: string);
    function GetConnected: boolean;
    procedure SetConnected(const Value: boolean);
    function CreateHandle: boolean;
    procedure DestroyHandle; 
  protected

  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    function Open: boolean;
    procedure Close;
    Function SetDTR(OnOff: Boolean): boolean;
    function SetRTS(OnOff: Boolean): boolean;
    property Connected: boolean read GetConnected write SetConnected;
    Property Port: string read FPort write SetPort;
  end;

  TDummyCom = class(TCustomDummyComPort)
    Property Port;
  end;

function EnumComPorts(Ports: TStrings): boolean;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples',[TDummyCom]);
end;

function EnumComPorts(Ports: TStrings): boolean;
var
  Reg: TRegistry;
  Str, TmpStr: TStringList;
  i: integer;
begin
  Result:= False;
  Ports.Clear;
  Str:= TStringList.Create;
  TmpStr:= TStringList.Create;
  Reg:= TRegistry.Create;
  Reg.RootKey:= HKEY_LOCAL_MACHINE;
  if Reg.OpenKeyReadOnly('\HARDWARE\DEVICEMAP\SERIALCOMM') then
  begin
    Reg.GetValueNames(Str);
    for i:= 0 to Str.Count - 1 do TmpStr.Add(Reg.ReadString(Str.Strings[i]));
    //TmpStr.Sort;
    Ports.Assign(TmpStr);
    Result:= True;
  end;
  Reg.Free;
  Str.Free;
  TmpStr.Free;
end;

{ TCustomDummyComPort }

procedure TCustomDummyComPort.Close;
begin
  DestroyHandle;
  FConnected:= False;
end;

constructor TCustomDummyComPort.Create(AOwner: TComponent);
begin
  inherited;
  FModemHandle:= INVALID_HANDLE_VALUE;
  FConnected:= False;
  FPort:= 'COM1';
end;

function TCustomDummyComPort.CreateHandle: boolean;
begin
  FModemHandle := CreateFile(
    PChar('\\.\' + FPort),
    GENERIC_READ or GENERIC_WRITE,
    0,
    nil,
    OPEN_EXISTING,
    FILE_FLAG_OVERLAPPED,
    0);

  Result:= FModemHandle <> INVALID_HANDLE_VALUE;
end;

destructor TCustomDummyComPort.Destroy;
begin
  Close;
  inherited;
end;

procedure TCustomDummyComPort.DestroyHandle;
begin
  if FModemHandle <> INVALID_HANDLE_VALUE then
  begin
    CloseHandle(FModemHandle);
    FModemHandle:= INVALID_HANDLE_VALUE;
  end;
end;

function TCustomDummyComPort.GetConnected: boolean;
begin
  Result:= FConnected;
end;

function TCustomDummyComPort.Open: boolean;
const
  dcb_Binary = $00000001;
var
  DCB: TDCB;
begin
  Result:= False;
  if FConnected then
  begin
    Result:= True;
    exit;
  end;
  if not CreateHandle then
    exit;

  //Apply DCB
  FillChar(DCB, SizeOf(TDCB), 0);
  DCB.DCBlength := SizeOf(TDCB);
  DCB.XonLim := 256;
  DCB.XoffLim := DCB.XonLim;
  DCB.EvtChar := #0;
  DCB.XonChar := #17;
  DCB.XoffChar := #19;
  DCB.Parity := NOPARITY;
  DCB.StopBits := ONESTOPBIT;
  DCB.BaudRate := CBR_9600;
  

  DCB.Flags := dcb_Binary; //dcb_binary
  DCB.ByteSize := 8;
  if SetCommState(FModemHandle, DCB) then
  begin
    FConnected:= True;
    Result:= True;
  end;
end;

procedure TCustomDummyComPort.SetConnected(const Value: boolean);
begin
  if Value then
    Open
  else
    Close;

end;

function TCustomDummyComPort.SetDTR(OnOff: Boolean): boolean;
var
  Act: DWORD;
begin
  Result:= False;
  if not FConnected then exit;

  if OnOff then
    Act := Windows.SETDTR
  else
    Act := Windows.CLRDTR;

  Result:= EscapeCommFunction(FModemHandle, Act);
end;

procedure TCustomDummyComPort.SetPort(const Value: string);
begin
  FPort := Value;
end;

function TCustomDummyComPort.SetRTS(OnOff: Boolean): boolean;
var
  Act: DWORD;
begin
  Result:= False;
  if not FConnected then exit;

  if OnOff then
    Act := Windows.SETRTS
  else
    Act := Windows.CLRRTS;

  Result:= EscapeCommFunction(FModemHandle, Act);
end;

end.
